﻿Imports System.IO
#Region "Template"
''' <summary>
''' Codes for FDA2D image format
''' </summary>
''' <remarks></remarks>
Public Enum CodeReadSave As Int32
    FUC = 100
    FI = 101
    FF = 102
    FD = 102
    FCX = 104
    F2UC = 200
    F2I = 201
    F2F = 202
    F2D = 203
    F2CX = 204
    F2UCRGB = 210
    F3UC = 300
    F3I = 301
    F3F = 302
    F3D = 303
    F3CX = 304
    F3UCRGB = 310
End Enum
''' <summary>
''' Template class for FDA2D images
''' </summary>
''' <typeparam name="t">Class embeded in each pixel of the image</typeparam>
''' <remarks></remarks>
Public Class FDA2DTemplate(Of t As New)
    Private _DT() As t
    Private _W As Int32
    Private _H As Int32
    Private _NP As Int32
    Public Sub New()
        Me._W = 1
        Me._H = 1
        Me._NP = 1
        ReDim Me._DT(0)
        Me._DT(0) = New t
    End Sub
    Protected Overrides Sub Finalize()
        Me._DT = Nothing
        MyBase.Finalize()
    End Sub
    ''' <summary>
    ''' Get or Set the value of a pixel
    ''' </summary>
    ''' <param name="y">y position of pixel</param>
    ''' <param name="x">x position of pixel</param>
    ''' <value>Value of the Pixel</value>
    ''' <returns>Value of the pixel</returns>
    ''' <remarks></remarks>
    Default Public Property Value(ByVal y As Int32, ByVal x As Int32) As t
        Get
            Return Me._DT(y * Me._W + x)
        End Get
        Set(ByVal value As t)
            Me._DT(y * Me._W + x) = value
        End Set
    End Property
    ''' <summary>
    ''' Get or Set the value of a pixel
    ''' </summary>
    ''' <param name="i">Index of the pixel</param>
    ''' <value>Value of the pixel</value>
    ''' <returns>Value of the pixel</returns>
    ''' <remarks></remarks>
    Default Public Property Value(ByVal i As Int32) As t
        Get
            Return Me._DT(i)
        End Get
        Set(ByVal value As t)
            Me._DT(i) = value
        End Set
    End Property
    ''' <summary>
    ''' Get the width of the FDA image
    ''' </summary>
    ''' <returns>Width</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property W() As Int32
        Get
            Return Me._W
        End Get
    End Property
    ''' <summary>
    ''' Get the height of the FDA image
    ''' </summary>
    ''' <returns>Height</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property H() As Int32
        Get
            Return Me._H
        End Get
    End Property
    ''' <summary>
    ''' Get the total number of pixels of the FDA image
    ''' </summary>
    ''' <returns>Total number of pixels</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property NumPix() As Int32
        Get
            Return Me._W * Me._H
        End Get
    End Property
    ''' <summary>
    ''' Resize the FDA image
    ''' </summary>
    ''' <param name="W">Width</param>
    ''' <param name="H">Height</param>
    ''' <remarks></remarks>
    Public Sub Resize(ByVal W As Int32, ByVal H As Int32)
        Dim i, np As Integer
        Me._W = W
        Me._H = H
        np = W * H - 1
        ReDim Me._DT(np)
        For i = 0 To np
            Me._DT(i) = New t
        Next
    End Sub
End Class
#End Region

#Region "FDA Types"
''' <summary>
''' Unsigned char (Byte) FDA image
''' </summary>
''' <remarks></remarks>
Public Class F2UC
    Inherits FDA2DTemplate(Of Byte)
    ''' <summary>
    ''' Load a FDA image from a file
    ''' </summary>
    ''' <param name="fname">File name</param>
    ''' <returns>true if success</returns>
    ''' <remarks>Returns false if the FDA image is not of Byte type</remarks>
    Public Function Load(ByVal fname As String) As Boolean
        Try
            If Not (File.Exists(fname)) Then
                Return False
            End If
            Dim Header(2) As Int32
            Dim i As Integer
            Using fs As New FileStream(fname, FileMode.Open)
                Dim br As New BinaryReader(fs)
                For i = 0 To 2
                    Header(i) = br.ReadInt32
                Next
                If Not (Header(0) = CodeReadSave.F2UC) Then
                    Return False
                End If
                Dim Wdata, Hdata As Int32
                Wdata = Header(1)
                Hdata = Header(2)
                Me.Resize(Wdata, Hdata)
                For i = 0 To Me.NumPix - 1
                    Me(i) = br.ReadByte
                Next
                fs.Close()
                Return True
            End Using
        Catch ex As Exception
            Return False
        End Try
    End Function
    ''' <summary>
    ''' Save a FDA image to a file
    ''' </summary>
    ''' <param name="fname">File name</param>
    ''' <returns>true if success</returns>
    ''' <remarks></remarks>
    Public Function Save(ByVal fname As String) As Boolean
        Try
            Dim code As Int32
            code = CodeReadSave.F2UC
            Dim Header(2) As Int32
            Header(0) = code
            Header(1) = Me.W
            Header(2) = Me.H
            Using fs As New FileStream(fname, FileMode.Create)
                fs.Lock(0, fs.Length)
                Dim br As New BinaryWriter(fs)
                Dim i As Int32
                For i = 0 To 2
                    br.Write(Header(i))
                Next
                For i = 0 To Me.NumPix - 1
                    br.Write(Me(i))
                Next
                fs.Unlock(0, fs.Length)
                fs.Close()
                Return True
            End Using
        Catch ex As Exception
            Return False
        End Try
    End Function
    ''' <summary>
    ''' Returns a new FDA image of same size
    ''' </summary>
    ''' <returns>FDA image (unsigned char)</returns>
    ''' <remarks>All pixels of the returned image are set to 0</remarks>
    Public Function CloneSize() As F2UC
        Dim val As New F2UC
        val.Resize(Me.W, Me.H)
        Return val
    End Function
End Class
''' <summary>
''' Int32 FDA image
''' </summary>
''' <remarks></remarks>
Public Class F2I
    Inherits FDA2DTemplate(Of Int32)
    ''' <summary>
    ''' Load a FDA image from a file
    ''' </summary>
    ''' <param name="fname">File name</param>
    ''' <returns>true if success</returns>
    ''' <remarks>Returns false if the FDA image is not of Int32 type</remarks>
    Public Function Load(ByVal fname As String) As Boolean
        Try
            If Not (File.Exists(fname)) Then
                Return False
            End If
            Dim Header(2) As Int32
            Dim i As Integer
            Using fs As New FileStream(fname, FileMode.Open)
                Dim br As New BinaryReader(fs)
                For i = 0 To 2
                    Header(i) = br.ReadInt32
                Next
                If Not (Header(0) = CodeReadSave.F2I) Then
                    Return False
                End If
                Dim Wdata, Hdata As Int32
                Wdata = Header(1)
                Hdata = Header(2)
                Me.Resize(Wdata, Hdata)
                For i = 0 To Me.NumPix - 1
                    Me(i) = br.ReadInt32
                Next
                fs.Close()
                Return True
            End Using
        Catch ex As Exception
            Return False
        End Try
    End Function
    ''' <summary>
    ''' Save a FDA image to a file
    ''' </summary>
    ''' <param name="fname">File name</param>
    ''' <returns>true if success</returns>
    ''' <remarks></remarks>
    Public Function Save(ByVal fname As String) As Boolean
        Try
            Dim code As Int32
            code = CodeReadSave.F2I
            Dim Header(2) As Int32
            Header(0) = code
            Header(1) = Me.W
            Header(2) = Me.H
            Using fs As New FileStream(fname, FileMode.Create)
                fs.Lock(0, fs.Length)
                Dim br As New BinaryWriter(fs)
                Dim i As Int32
                For i = 0 To 2
                    br.Write(Header(i))
                Next
                For i = 0 To Me.NumPix - 1
                    br.Write(Me(i))
                Next
                fs.Unlock(0, fs.Length)
                fs.Close()
                Return True
            End Using
        Catch ex As Exception
            Return False
        End Try
    End Function
    ''' <summary>
    ''' Returns a new FDA image of same size
    ''' </summary>
    ''' <returns>FDA image (unsigned char)</returns>
    ''' <remarks>All pixels of the returned image are set to 0</remarks>
    Public Function CloneSize() As F2I
        Dim val As New F2I
        val.Resize(Me.W, Me.H)
        Return val
    End Function

End Class
''' <summary>
''' Double FDA image
''' </summary>
''' <remarks></remarks>
Public Class F2D
    Inherits FDA2DTemplate(Of Double)
    ''' <summary>
    ''' Load a FDA image from a file
    ''' </summary>
    ''' <param name="fname">File name</param>
    ''' <returns>true if success</returns>
    ''' <remarks>Returns false if the FDA image is not of Double type</remarks>
    Public Function Load(ByVal fname As String) As Boolean
        Try
            If Not (File.Exists(fname)) Then
                Return False
            End If
            Dim Header(2) As Int32
            Dim i As Integer
            Using fs As New FileStream(fname, FileMode.Open)
                Dim br As New BinaryReader(fs)
                For i = 0 To 2
                    Header(i) = br.ReadInt32
                Next
                If Not (Header(0) = CodeReadSave.F2D) Then
                    Return False
                End If
                Dim Wdata, Hdata As Int32
                Wdata = Header(1)
                Hdata = Header(2)
                Me.Resize(Wdata, Hdata)
                For i = 0 To Me.NumPix - 1
                    Me(i) = br.ReadDouble
                Next
                fs.Close()
                Return True
            End Using
        Catch ex As Exception
            Return False
        End Try
    End Function
    ''' <summary>
    ''' Save a FDA image to a file
    ''' </summary>
    ''' <param name="fname">File name</param>
    ''' <returns>true if success</returns>
    ''' <remarks></remarks>
    Public Function Save(ByVal fname As String) As Boolean
        Try
            Dim code As Int32
            code = CodeReadSave.F2D
            Dim Header(2) As Int32
            Header(0) = code
            Header(1) = Me.W
            Header(2) = Me.H
            Using fs As New FileStream(fname, FileMode.Create)
                fs.Lock(0, fs.Length)
                Dim br As New BinaryWriter(fs)
                Dim i As Int32
                For i = 0 To 2
                    br.Write(Header(i))
                Next
                For i = 0 To Me.NumPix - 1
                    br.Write(Me(i))
                Next
                fs.Unlock(0, fs.Length)
                fs.Close()
                Return True
            End Using
        Catch ex As Exception
            Return False
        End Try
    End Function
    ''' <summary>
    ''' Returns a new FDA image of same size
    ''' </summary>
    ''' <returns>FDA image (unsigned char)</returns>
    ''' <remarks>All pixels of the returned image are set to 0</remarks>
    Public Function CloneSize() As F2D
        Dim val As New F2D
        val.Resize(Me.W, Me.H)
        Return val
    End Function

End Class
''' <summary>
''' Triplet of Bytes for RGB images
''' </summary>
''' <remarks></remarks>
Public Class RGBUC
    Private _R As Byte
    Private _G As Byte
    Private _B As Byte
    Public Sub New()
        _R = 0
        _G = 0
        _B = 0
    End Sub
    ''' <summary>
    ''' Set or Get the red channel
    ''' </summary>
    ''' <value>Value of the red channel</value>
    ''' <returns>Value of the red channel</returns>
    ''' <remarks></remarks>
    Public Property R() As Byte
        Get
            Return _R
        End Get
        Set(ByVal value As Byte)
            _R = value
        End Set
    End Property
    ''' <summary>
    ''' Set or Get the green channel
    ''' </summary>
    ''' <value>Value of the green channel</value>
    ''' <returns>Value of the green channel</returns>
    ''' <remarks></remarks>
    Public Property G() As Byte
        Get
            Return _G
        End Get
        Set(ByVal value As Byte)
            _G = value
        End Set
    End Property
    ''' <summary>
    ''' Set or Get the blue channel
    ''' </summary>
    ''' <value>Value of the blue channel</value>
    ''' <returns>Value of the blue channel</returns>
    ''' <remarks></remarks>
    Public Property B() As Byte
        Get
            Return _B
        End Get
        Set(ByVal value As Byte)
            _B = value
        End Set
    End Property
End Class
''' <summary>
''' 24 Byte  RGB FDA image
''' </summary>
''' <remarks></remarks>
Public Class F2UCRGB
    Inherits FDA2DTemplate(Of RGBUC)
    ''' <summary>
    ''' Load a FDA image from a file
    ''' </summary>
    ''' <param name="fname">File name</param>
    ''' <returns>true if success</returns>
    ''' <remarks>Returns false if the FDA image is not of RGB (3 Bytes) type</remarks>
    Public Function Load(ByVal fname As String) As Boolean
        Try
            If Not (File.Exists(fname)) Then
                Return False
            End If
            Dim Header(2) As Int32
            Dim i As Integer
            Using fs As New FileStream(fname, FileMode.Open)
                Dim br As New BinaryReader(fs)
                For i = 0 To 2
                    Header(i) = br.ReadInt32
                Next
                If Not (Header(0) = CodeReadSave.F2UCRGB) Then
                    Return False
                End If
                Dim Wdata, Hdata, NP As Int32
                Wdata = Header(1)
                Hdata = Header(2)
                NP = Wdata * Hdata
                Me.Resize(Wdata, Hdata)
                '   Red
                For i = 0 To NP - 1
                    Me(i).R = br.ReadByte
                Next
                '   Green
                For i = 0 To NP - 1
                    Me(i).G = br.ReadByte
                Next
                '   Blue
                For i = 0 To NP - 1
                    Me(i).B = br.ReadByte
                Next
                fs.Close()
                Return True
            End Using
        Catch ex As Exception
            Return False
        End Try
    End Function
    ''' <summary>
    ''' Save a FDA image to a file
    ''' </summary>
    ''' <param name="fname">File name</param>
    ''' <returns>true if success</returns>
    ''' <remarks></remarks>
    Public Function Save(ByVal fname As String) As Boolean
        Try
            Dim code, NP As Int32
            code = CodeReadSave.F2UCRGB
            Dim Header(2) As Int32
            Header(0) = code
            Header(1) = Me.W
            Header(2) = Me.H
            NP = Me.NumPix
            Using fs As New FileStream(fname, FileMode.Create)
                fs.Lock(0, fs.Length)
                Dim br As New BinaryWriter(fs)
                Dim i As Int32
                For i = 0 To 2
                    br.Write(Header(i))
                Next
                '   Red
                For i = 0 To NP - 1
                    br.Write(Me(i).R)
                Next
                '   Green
                For i = 0 To NP - 1
                    br.Write(Me(i).G)
                Next
                '   Blue
                For i = 0 To NP - 1
                    br.Write(Me(i).B)
                Next
                fs.Unlock(0, fs.Length)
                fs.Close()
                Return True
            End Using
        Catch ex As Exception
            Return False
        End Try
    End Function
    ''' <summary>
    ''' Returns a new FDA image of same size
    ''' </summary>
    ''' <returns>FDA image (unsigned char)</returns>
    ''' <remarks>All pixels of the returned image are set to 0</remarks>
    Public Function CloneSize() As F2UCRGB
        Dim val As New F2UCRGB
        val.Resize(Me.W, Me.H)
        Return val
    End Function

End Class
Public Class Complex
    Private _Re As Double
    Private _Im As Double
    Public Sub New()
        _Re = 0.0
        _Im = 0.0
    End Sub
    Public Property Re() As Double
        Get
            Return Me._Re
        End Get
        Set(ByVal value As Double)
            Me._Re = value
        End Set
    End Property
    Public Property Im() As Double
        Get
            Return Me._Im
        End Get
        Set(ByVal value As Double)
            Me._Im = value
        End Set
    End Property
    Public ReadOnly Property Modulus() As Double
        Get
            Return Math.Sqrt(Me._Re * Me._Re + Me._Im * Me._Im)
        End Get
    End Property
    Public ReadOnly Property Phase() As Double
        Get
            If Me._Re = 0.0 And Me._Im = 0.0 Then
                Return 0.0
            Else
                If (Me._Im >= 0.0) Then
                    Return Math.Acos(Me._Re / Me.Modulus)
                Else
                    Return 2.0 * Math.PI - Math.Acos(Me._Re / Me.Modulus)
                End If
            End If
        End Get
    End Property
End Class
Public Class F2CX
    Inherits FDA2DTemplate(Of Complex)
    ''' <summary>
    ''' Load a FDA image from a file
    ''' </summary>
    ''' <param name="fname">File name</param>
    ''' <returns>true if success</returns>
    ''' <remarks>Returns false if the FDA image is not of Complex type</remarks>
    Public Function Load(ByVal fname As String) As Boolean
        Try
            If Not (File.Exists(fname)) Then
                Return False
            End If
            Dim Header(2) As Int32
            Dim i As Integer
            Using fs As New FileStream(fname, FileMode.Open)
                Dim br As New BinaryReader(fs)
                For i = 0 To 2
                    Header(i) = br.ReadInt32
                Next
                If Not (Header(0) = CodeReadSave.F2CX) Then
                    Return False
                End If
                Dim Wdata, Hdata As Int32
                Wdata = Header(1)
                Hdata = Header(2)
                Me.Resize(Wdata, Hdata)
                For i = 0 To Me.NumPix - 1
                    Me(i).Re = br.ReadDouble
                Next
                For i = 0 To Me.NumPix - 1
                    Me(i).Im = br.ReadDouble
                Next
                fs.Close()
                Return True
            End Using
        Catch ex As Exception
            Return False
        End Try
    End Function
    ''' <summary>
    ''' Save a FDA image to a file
    ''' </summary>
    ''' <param name="fname">File name</param>
    ''' <returns>true if success</returns>
    ''' <remarks></remarks>
    Public Function Save(ByVal fname As String) As Boolean
        Try
            Dim code As Int32
            code = CodeReadSave.F2CX
            Dim Header(2) As Int32
            Header(0) = code
            Header(1) = Me.W
            Header(2) = Me.H
            Using fs As New FileStream(fname, FileMode.Create)
                fs.Lock(0, fs.Length)
                Dim br As New BinaryWriter(fs)
                Dim i As Int32
                For i = 0 To 2
                    br.Write(Header(i))
                Next
                For i = 0 To Me.NumPix - 1
                    br.Write(Me(i).Re)
                Next
                For i = 0 To Me.NumPix - 1
                    br.Write(Me(i).Im)
                Next
                fs.Unlock(0, fs.Length)
                fs.Close()
                Return True
            End Using
        Catch ex As Exception
            Return False
        End Try
    End Function
    ''' <summary>
    ''' Returns a new FDA image of same size
    ''' </summary>
    ''' <returns>FDA image (unsigned char)</returns>
    ''' <remarks>All pixels of the returned image are set to 0</remarks>
    Public Function CloneSize() As F2CX
        Dim val As New F2CX
        val.Resize(Me.W, Me.H)
        Return val
    End Function
End Class
#End Region












